package com.muyu.cloud.vehicle.gateway.config;

import lombok.extern.log4j.Log4j2;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.ClientHttpRequestFactory;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

/**
 * rest配置类
 */
@Log4j2
@Configuration
public class RestTemplateConfig {
    @Bean
    public RestTemplate restTemplate(ClientHttpRequestFactory factory){
        return new RestTemplate(factory);
    }

    /**
     * 创建并配置客户端HTTP请求工厂实例。
     * 注意：这里设置了连接和读取数据的超时时间，分别设为15秒和5秒。
     * 这些值可以根据实际网络状况和应用需求进行调整。
     *
     * @return 配置好的客户端请求工厂对象
     */
    @Bean
    public ClientHttpRequestFactory simpleClientHttpRequestFactory(){
        try{
            SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
            //设置读取超时时间为5秒
            factory.setReadTimeout(5000);
            //设置连接超时时间为15秒
            factory.setConnectTimeout(15000);
            return factory;
        } catch (Exception e) {
            //处理创建工厂或设置超时时间可能出现的异常
            log.info("创建工厂失败：" + e.getMessage());
            throw new RuntimeException("初始化HTTP请求失败",e);
        }
    }
}
